local type, ipairs = type, ipairs
local concat = table.concat
local print = print
local require, unpack = require, unpack

module ("nsg")

local function getHostAndPort(hp)
  if type(hp) == 'string' then
    return hp:match("(.*):(.*)")
  end
end

local function pipeRead(pipe, tmo)
  pipe:settimeout(tmo)
  local data, emsg, partial = pipe:receive(2048)
  if data then
    return data
  end
  if partial and #partial > 0 then
    return partial
  end
  return nil, emsg
end

function spy(conf) 
  local socket = require"socket"
  local host, port = getHostAndPort(conf.server)
  
  while true do
    local out = {}
    for _, v in ipairs(conf) do
      local pipe, err = socket.pipe(unpack(v.script))
      if pipe then
        local res, err = pipeRead(pipe, conf.waittime or 5)
        pipe:close()
        local restable
        if res then
          if v.match then
            restable = {res:match(v.match)}
          else
            restable = {res}
          end
          local parms = type(v.parms) == 'table' and v.parms or {v.parms}
          for i, prm in ipairs(parms) do
            out[#out+1] = ("[%q]"):format(prm)..'='..("%q"):format(restable[i])
          end
        else
          print(("Can't execute script(%s)"):format(err))
        end
      else
        print(("Can't execute script(%s)"):format(err))
      end
    end
    out = concat(out, ',')
    print(out)
    local con, err = socket.connect(host, port)
    if con then
      con:send(out)
      con:close()
    else
      print(("Can't connect %s:%s(%s)"):format(host, port, err))
    end
    socket.sleep(conf.interval)
  end
end
